@@ -27,8 +27,7 @@
 (use-modules (nyacc lang c99 munge-base))
 (use-modules (nyacc lang c99 cpp))
 (use-modules (nyacc lang c99 util))
-(use-modules (nyacc lang c99 ffi-help))
-(use-modules (nyacc lang arch-info))
+(use-modules (nyacc foreign arch-info))
 (use-modules (nyacc lang sx-util))
 (use-modules (nyacc lang util))
 (use-modules (nyacc lex))
@@ -112,30 +111,6 @@
     (pp node)
     (sf "(source-properties node) => ~S\n" (source-properties node))))
 
-(when #f                               ; bug #60474
-  (let* ((code "const int x = 1;\n")
-         (tree (parse-string code #:mode 'code))
-         (udict (c99-trans-unit->udict tree))
-         (udecl (assoc-ref udict "x"))
-         (specl (sx-ref udecl 1))
-         (declr (sx-ref udecl 2)))
-    (pp udecl)
-    (call-with-values (lambda () (cleanup-udecl specl declr))
-      (lambda (specl declr) (pp `(udecl ,specl ,declr))))))
-
-;; bug #60474
-(when #f
-  (let* ((code "const int x = 1;\n")
-         (tree (parse-string code #:mode 'code))
-         (udict (c99-trans-unit->udict tree))
-         (udecl (assoc-ref udict "x"))
-         (specl (sx-ref udecl 1))
-         (declr (sx-ref udecl 2)))
-    (pp udecl)
-    (call-with-values (lambda () (cleanup-udecl specl declr))
-      (lambda (specl declr)
-        (pp `(udecl ,specl ,declr))))))
-
 (when #f
   (let* ((code "typedef int foo_t; foo_t foo[] = { 1, 2, 3, 4 };" )
          (tree (parse-string code #:mode 'decl))
@@ -255,16 +230,6 @@
          (foo-t '(type-name (decl-spec-list (type-spec (typename "foo_t"))))))
     (pp (find-types foo-t udict '("let_t")))))
 
-;; bug 57949
-(when #f
-  (let* ((code "struct foo { void bar(void); };\n")
-         (tree (parse-string code))
-         (tree (remove-comments tree))
-         (udict (c99-trans-unit->udict tree))
-         (udecl (assoc-ref udict '(struct . "foo")))
-         (fdecl (fh-cnvt-udecl udecl udict))
-         (sdecl (with-input-from-string fdecl read)))
-    (display fdecl)))
 
 ;; bug #63604
 (when #f
@@ -375,9 +340,30 @@
 (when #f
   ;; bug 65849
   (let* ((code (string-append
-                "#warning \"warn me!\"\n" 
+                "#warning \"warn me!\"\n"
                 "#warning \"oops\"\n"))
          (tree (parse-string code)))
     #t))
 
+(when #f
+  (let* ((code "int foo(int x, float, double *);")
+         (tree (parse-string code))
+         (params (sx-tail (sx-ref* tree 1 2 1 1 2)))
+         )
+    (for-each
+     (lambda (p) (pp (reify-udecl p)))
+      params)))
+
+(when #f
+  (let* ((code "typedef enum foo { FOO = 1 } foo_t;\nfoo_t x;\n")
+         (tree (parse-string code))
+         (udict (c99-trans-unit->udict tree))
+         (udecl (sx-ref* tree 2))
+         (xdecl (expand-typerefs udecl udict '((enum . "foo"))))
+         )
+    ;;(pp udict)
+    ;;(pp udecl)
+    (pp xdecl)
+    ))
+
 ;; --- last line ---
