@@ -108,17 +108,21 @@
             cpointer-type cpointer-mtype
             carray-type carray-length
             cenum-vald cenum-symd
-            cfunction-proc->ptr cfunction-ptr->proc
-            )
+            cfunction-proc->ptr cfunction-ptr->proc)
   #:use-module (ice-9 match)
   #:use-module (ice-9 format)
   #:use-module ((srfi srfi-1) #:select (fold xcons))
   #:use-module (srfi srfi-9)
   #:use-module (srfi srfi-9 gnu)
   #:use-module (rnrs bytevectors)
-  #:use-module (system foreign)
+  #:use-module ((system foreign)
+                #:select (%null-pointer
+                          make-pointer pointer? pointer-address
+                          scm->pointer string->pointer
+                          pointer->bytevector bytevector->pointer
+                          int8 uint8 int16 uint16 int32 uint32 int64 uint64
+                          float double))
   #:use-module (nyacc foreign arch-info))
-(export %cpointer-type)
 
 (use-modules (ice-9 pretty-print))
 (define (pperr exp) (pretty-print exp (current-error-port)))
@@ -302,7 +306,6 @@
   (bv cdata-bv)                         ; bvec
   (ix cdata-ix)                         ; index
   (ct cdata-ct))                        ; type
-(export %make-cdata) ;; needed?
 
 (set-record-type-printer! <cdata>
   (lambda (data port)
@@ -386,7 +389,6 @@
   (with-arch arch
     (map (lambda (name) (cons name (make-cbase name)))
          base-type-symbol-list)))
-(export make-cbase-map)
 
 (define cbase-symbols
   '(s8 u8 s16 s32 s64 i128 u16 u32 u64 u128 f16 f32 f64 f128
@@ -605,11 +607,11 @@
 ;; @end example
 ;; If @var{packed} is @code{#t} the size wil be smallest that can hold it.
 ;; @end deffn
-(define* (cenum enum-list #:optional basetype)
-  "- Procedure: cenum enum-list [basetype]
+(define* (cenum enum-list #:optional packed)
+  "- Procedure: cenum enum-list [packed]
      ENUM-LIST is a list of name or name-value pairs
           (cenum '((a 1) b (c 4))
-     If SHORT is ‘#t’ the size wil be smallest that can hold it."
+     If PACKED is ‘#t’ the size wil be smallest that can hold it."
   (define (short-mtype mn mx)
     (if (< 0 mn)
         (cond
@@ -627,7 +629,7 @@
     (if (null? enl)
         (let* ((mx (cdar nvl)) (nvl (reverse nvl)) (mn (cdar nvl))
                (vnl (map (lambda (p) (cons (cdr p) (car p))) nvl))
-               (mtype (if short (short-mtype mn mx) (mtypeof-basetype 'int))))
+               (mtype (if packed (short-mtype mn mx) (mtypeof-basetype 'int))))
           (%make-ctype (sizeof-mtype mtype) (alignof-mtype mtype)
                        'enum (%make-cenum mtype vnl nvl) #f))
         (match (car enl)
@@ -1273,6 +1275,14 @@
           (else `(cpointer ,(cnvt (cpointer-type info))))))
         ((array)
          `(carray ,(cnvt (carray-type info)) ,(carray-length info)))
+        ((enum)
+         (cond
+          ((ctype-name type) => identity)
+          (else `(enum ...))))
+        ((function)
+         (cond
+          ((ctype-name type) => identity)
+          (else `(function ...))))
         (else (error "pretty-print-ctype: needs work:" (ctype-kind type))))))
   (pretty-print (cnvt type) port))
 
